
// Agent.fsi - Aivika

// Copyright (c) 2009, 2010  David Sorokin. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials
//     provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Author: David Sorokin <david.sorokin@gmail.com>
//

namespace Maritegra.Aivika
        
open System        
        
type InvalidAgentException =
    inherit Exception
    
    new: agent:Agent * message:string -> InvalidAgentException
    member Agent: Agent
        
and [<AbstractClass>] Agent =

    new: e:Env -> Agent
    member Env: Env
    
    member StartState: State
    member FinalState: State
    
    member State: Dynamics<State>
    member SetState: st:State -> Dynamics<unit>
    member SetInitialState: st:State -> Dynamics<unit>
    
    abstract Activate: Dynamics<unit>
    abstract Deactivate: Dynamics<unit>
    
    default Activate: Dynamics<unit>
    default Deactivate: Dynamics<unit>
    
    member Start: t:float -> Dynamics<unit>
    member StartD: t:Dynamics<float> -> Dynamics<unit>
    
    member SetTimeout: t:float * action:Dynamics<unit> -> Dynamics<unit>
    member SetTimeoutD: t:Dynamics<float> * action:Dynamics<unit> -> Dynamics<unit>
    
    member SetTimer: t:float * action:Dynamics<unit> -> Dynamics<unit>
    member SetTimerD: t:Dynamics<float> * action:Dynamics<unit> -> Dynamics<unit>
        
and [<AbstractClass>] State =

    internal new: agent:Agent * parent:State option -> State
    new: parent:State -> State
    new: agent:Agent -> State
    
    member Agent: Agent
    member Parent: State option
    
    abstract Activate: Dynamics<unit>
    abstract Deactivate: Dynamics<unit>
    
    default Activate: Dynamics<unit>
    default Deactivate: Dynamics<unit>
    
    member SetTimeout: t:float * action:Dynamics<unit> -> Dynamics<unit>
    member SetTimeoutD: t:Dynamics<float> * action:Dynamics<unit> -> Dynamics<unit>
    
    member SetTimer: t:float * action:Dynamics<unit> -> Dynamics<unit>
    member SetTimerD: t:Dynamics<float> * action:Dynamics<unit> -> Dynamics<unit>