
// Discrete.fsi - Aivika

// Copyright (c) 2009, 2010  David Sorokin. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials
//     provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Author: David Sorokin <david.sorokin@gmail.com>
//

namespace Maritegra.Aivika

type Env =
    new: unit -> Env
    member Run: Dynamics<unit>
    member Dispatch: t:Dynamics<float> * c:Dynamics<unit> -> Dynamics<unit>
    member DispatchD: t:Dynamics<float> * c:(Dynamics<unit -> unit>) -> Dynamics<unit>
    
and [<AbstractClass>] Process =
    new: e:Env -> Process
    member Env: Env
    member Start: t:float -> Dynamics<unit>
    member StartD: t:Dynamics<float> -> Dynamics<unit>
    member Hold: t:float -> DynamicsCont<unit>
    member HoldD: t:Dynamics<float> -> DynamicsCont<unit>
    abstract Activate: DynamicsCont<unit>
    member IsPassivated: DynamicsCont<bool>
    member Passivate: DynamicsCont<unit>
    member Reactivate: DynamicsCont<unit>
    
and Resource =
    new: e:Env * number:int -> Resource
    member Env: Env
    member N: DynamicsCont<int>
    member Request: DynamicsCont<unit>
    member Release: DynamicsCont<unit>
    
and Var<'a> =
    new: e:Env * i:Dynamics<'a> -> Var<'a>
    new: e:Env * i:'a -> Var<'a>
    member Env: Env
    member Value: Dynamics<'a>
    member SetValue: a:'a -> Dynamics<unit>
    
and MemorylessVar<'a> =
    new: e:Env * i:Dynamics<'a> -> MemorylessVar<'a>
    new: e:Env * i:'a -> MemorylessVar<'a>
    member Env: Env
    member Value: Dynamics<'a>
    member SetValue: a:'a -> Dynamics<unit>

[<CompilationRepresentation (CompilationRepresentationFlags.ModuleSuffix)>]
module Var =

    [<CompiledName ("Env")>]
    val env: Var<'a> -> Env
    
    [<CompiledName ("Value")>]
    val value: Var<'a> -> Dynamics<'a>
    
    [<CompiledName ("SetValue")>]
    val setValue: 'a -> Var<'a> -> Dynamics<unit>
    
    [<CompiledName ("IncValue")>]
    val inline incValue: 'a -> Var<'a> -> Dynamics<unit> when 'a: (static member (+): 'a * 'a -> 'a)
    
    [<CompiledName ("DecValue")>]
    val inline decValue: 'a -> Var<'a> -> Dynamics<unit> when 'a: (static member (-): 'a * 'a -> 'a)

[<CompilationRepresentation (CompilationRepresentationFlags.ModuleSuffix)>]
module MemorylessVar =

    [<CompiledName ("Env")>]
    val env: MemorylessVar<'a> -> Env
    
    [<CompiledName ("Value")>]
    val value: MemorylessVar<'a> -> Dynamics<'a>
    
    [<CompiledName ("SetValue")>]
    val setValue: 'a -> MemorylessVar<'a> -> Dynamics<unit>
    
    [<CompiledName ("IncValue")>]
    val inline incValue: 'a -> MemorylessVar<'a> -> Dynamics<unit> when 'a: (static member (+): 'a * 'a -> 'a)
    
    [<CompiledName ("DecValue")>]
    val inline decValue: 'a -> MemorylessVar<'a> -> Dynamics<unit> when 'a: (static member (-): 'a * 'a -> 'a)
        