﻿
(* This simple example illustrates the main idea of that how 
   a thread-safe iterative process can be implemented with help of 
   the DynamicsCont workflow. For example, then the data generated 
   by such a process can be used in System Dynamics' model during
   different simulations launched parallelly. 
   
   This model generates Fibonacci numbers for simplicity. *)

open System

open Maritegra.Aivika
open Maritegra.Aivika.SD

let specs = {

    StartTime=0.0; StopTime=1000.0; DT=1.0; 
    Method=RungeKutta4; Randomness=SimpleRnd
}

let env = Env ()

let fib = Var<bigint> (env, bigint 0)

let proc =
    { new Process (env) with
    
        override x.Activate = dynamicscont {
        
            let next a = dynamicscont {
            
                do! DynamicsCont.lift <| fib.SetValue (a)
                do! x.HoldD (dt)
            }
            
            let rec loop a b = dynamicscont {
            
                let c = a + b
            
                do! next c
                do! loop b c
            }
            
            do! next 0I
            do! next 1I
            do! loop 0I 1I
        }
    }
    
let starter = proc.StartD (starttime)   // launch the process

let system = dynamics {

    do! starter         // launch the process
    do! env.Run         // launch the environment
    
    return! fib.Value
}

[<EntryPoint>]
let main args =

    let results = Dynamics.run system specs

    Console.WriteLine ("Fibonacci Number")
    for r in results do
        Console.WriteLine (r)

    0