﻿
(* A simple chemical reaction. The model is described in 
   the documentation (5-Minute Tutorial) of 
   Berkeley&Madonna [http://www.berkeleymadonna.com/].

   The equations are as follows:

   STARTTIME = 0
   STOPTIME = 10
   DT = 0.02

   d/dt (A) = - ka*A
   d/dt (B) =ka*A - kb*B 
   d/dt (C)  = kb*B
   init A =100
   init B = 0
   init C = 0
   ka =1
   kb =1 *)

#nowarn "40"

open Maritegra.Aivika
open Maritegra.Aivika.SD

let specs = {

    StartTime=0.0; StopTime=10.0; DT=0.02; 
    Method=RungeKutta4; Randomness=StrongRnd
}

let rec a = integ (lazy (-ka * a)) (eta 100.0)
and b = integ (lazy (ka*a - kb*b)) (eta 0.0)
and c = integ (lazy (kb*b)) (eta 0.0)
and ka = 1.0
and kb = 1.0

let system = Dynamics.ofArray [| time; a; b; c |]

[<EntryPoint>]
let main args =

    let results = Dynamics.run system specs
    
    printfn "Time,A,B,C"
    
    for r in results do
        printfn "%f,%f,%f,%f" r.[0] r.[1] r.[2] r.[3]
    
    0